@@ -1,5 +1,6 @@
 /*
- * Copyright 1996-2003 Hans Reiser
+ * Copyright 1996-2003 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
 
 #include "fsck.h"
@@ -31,7 +32,14 @@ reiserfs_bitmap_t * bad_unfm_bitmap;
 #define pass0_mark_bad_unfm(block) __mark (bad_unfm, block)
 #define pass0_unmark_bad_unfm(block) __unmark (bad_unfm, block)
 
-
+static int correct_direct_item_offset (__u64 offset, int format) {
+    if (format == KEY_FORMAT_2) {
+        return (offset && ((offset - 1) % 8 == 0));
+    } else {
+	return (offset);
+    }
+    return 0;
+}
 
 /* bitmaps which are built on pass 0 and are used on pass 1 */
 static void make_aux_bitmaps (reiserfs_filsys_t * fs)
@@ -233,7 +241,7 @@ static int verify_directory_item (reiserfs_filsys_t * fs, struct buffer_head * b
     struct reiserfs_de_head * deh;
     char * name;
     int name_len;
-    int bad;
+    int bad, lost_found;
     int i, j;
     char buf[4096];
     int dirty;
@@ -477,7 +485,7 @@ static int verify_directory_item (reiserfs_filsys_t * fs, struct buffer_head * b
     }
 #endif
 
-    bad = 0;
+    bad = lost_found = 0;
     tmp = *ih;
 
     /* mark enries of /lost+found as bad */
@@ -497,6 +505,7 @@ static int verify_directory_item (reiserfs_filsys_t * fs, struct buffer_head * b
 	
 	/* entry in lost+found */
 //	printf ("%s - will be deleted\n", buf);
+	lost_found++;
 	mark_de_bad_offset (deh);
     }
 
@@ -515,9 +524,9 @@ static int verify_directory_item (reiserfs_filsys_t * fs, struct buffer_head * b
 	}
     }
     
-    if (bad == get_ih_entry_count (&tmp)) {
+    if (bad == get_ih_entry_count (&tmp) && lost_found != bad) {	
 	fsck_log ("%s: block %lu, item %H: All entries were deleted from the directory\n", 
-	    __FUNCTION__, bh->b_blocknr, &tmp);
+	    __FUNCTION__, bh->b_blocknr, &tmp);	
 	return -1;
     }
 
@@ -529,9 +538,9 @@ static int verify_directory_item (reiserfs_filsys_t * fs, struct buffer_head * b
 	mark_buffer_dirty (bh);
     }
     
-    if (bad)
+    if (bad > lost_found)
 	fsck_log ("pass0: block %lu, item %H: %d entries were deleted\n",
-		  bh->b_blocknr, &tmp, bad);
+		  bh->b_blocknr, &tmp, bad - lost_found);
 	
     return 0;
 
@@ -550,7 +559,7 @@ static __inline__ int does_it_fit_into_dev (__u64 offset, __u64 fs_size) {
 
 static int is_wrong_short_key (struct key * key) {
     if (get_key_dirid (key) == 0 || get_key_objectid (key) == 0 || get_key_objectid (key) == 1 ||
-	get_key_dirid (key) == -1 || get_key_objectid (key) == -1 ||
+	get_key_dirid (key) == ~(__u32)0 || get_key_objectid (key) == ~(__u32)0 ||
 	get_key_dirid (key) == get_key_objectid (key) ||
 	(get_key_dirid (key) == 1 && get_key_objectid (key) != 2) ||
 	(get_key_dirid (key) != 1 && get_key_objectid (key) == 2) )
@@ -559,6 +568,55 @@ static int is_wrong_short_key (struct key * key) {
     return 0;
 }
 
+/* 1 if some of fields in the block head of bh look bad */
+int leaf_structure_check (reiserfs_filsys_t * fs, struct buffer_head * bh) {
+    struct block_head * blkh;
+    int free_space, counted;
+
+    blkh = B_BLK_HEAD (bh);
+    
+    if (!is_leaf_block_head(bh->b_data)) {
+	/* We should not get here on rebuild. */
+	fsck_log ("block %lu: The block does not look like a leaf.\n", bh->b_blocknr);
+        one_more_corruption (fs, FATAL);
+	return 1;
+    }
+
+    counted = leaf_count_ih(bh->b_data, bh->b_size);
+
+    if (counted < get_blkh_nr_items (blkh)) {
+	fsck_log ("block %lu: The number of items (%lu) is incorrect, should be (%lu)",
+		bh->b_blocknr, get_blkh_nr_items(blkh), counted);
+	if (fsck_mode(fs) == FSCK_REBUILD) {
+	    set_blkh_nr_items(blkh, counted);	    
+	    fsck_log (" - corrected\n");
+	    mark_buffer_dirty (bh);
+	} else {
+	    fsck_log ("\n");
+	    one_more_corruption (fs, FATAL);
+	    return 1;
+	}
+    }
+    
+    free_space = leaf_free_space_estimate(bh->b_data, bh->b_size);
+    if (get_blkh_free_space (blkh) != free_space) {
+	fsck_log ("block %lu: The free space (%lu) is incorrect, should be (%lu)",
+		bh->b_blocknr, get_blkh_free_space (blkh), free_space);
+	
+	if (fsck_mode(fs) != FSCK_CHECK && fsck_mode(fs) != FSCK_AUTO) {
+	    set_blkh_free_space (blkh, free_space);	    
+	    fsck_log (" - corrected\n");
+	    mark_buffer_dirty (bh);
+	} else {
+	    fsck_log ("\n");
+	    one_more_corruption (fs, FIXABLE);
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 /* do this on pass 0 with every leaf marked used */
 
 /* FIXME: we can improve fixing of broken keys: we can ssfe direct items which
@@ -566,21 +624,25 @@ static int is_wrong_short_key (struct key * key) {
 static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 				struct buffer_head * bh)
 {
-    int i, j;
-    struct item_head * ih;
-    __u32 * ind_item;
-    __u64 offset;
-    unsigned long unfm_ptr;
-    int dirty = 0;
-    int nr_items;
     int file_format = KEY_FORMAT_UNDEFINED;
+    struct item_head * ih;
+    
+    __u32 * ind_item;
+    __u64 fs_size;
+    __u64 offset;
     int symlnk = 0;
     int bad_order;
-    __u64 fs_size;
+    
+    unsigned long unfm_ptr;
+//    unsigned int nr_items;
+    int i, j, nr_items;
+    int dirty = 0;
+
+    leaf_structure_check(fs, bh);
 
     /* Delete all safe links. */
     for (i = get_blkh_nr_items (B_BLK_HEAD (bh)) - 1; i >= 0; i--) {
-	if (get_key_dirid (&B_N_PITEM_HEAD (bh, i)->ih_key) == -1) {
+	if (get_key_dirid (&B_N_PITEM_HEAD (bh, i)->ih_key) == ~(__u32)0) {
 	    delete_item (fs, bh, i);
 	}
     }
@@ -903,7 +965,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 		    if (get_ih_item_len (ih - 1) == SD_SIZE) {
 			/* stat data is new, therefore this item is new too */
 			set_offset (KEY_FORMAT_2, &(ih->ih_key), 1);
-			if (get_ih_entry_count (ih) != 0xffff)
+			if ((get_ih_entry_count (ih) != 0xffff) && (get_ih_item_len (ih) % 4 == 0))
 			    set_type (KEY_FORMAT_2, &(ih->ih_key), TYPE_INDIRECT);
 			else
 			    set_type (KEY_FORMAT_2, &(ih->ih_key), TYPE_DIRECT);
@@ -911,7 +973,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 		    } else {
 			/* stat data is old, therefore this item is old too */
 			set_offset (KEY_FORMAT_1, &(ih->ih_key), 1);
-			if (get_ih_entry_count (ih) != 0xffff && (get_ih_item_len (ih) % 4 == 0))
+			if ((get_ih_entry_count (ih) != 0xffff) && (get_ih_item_len (ih) % 4 == 0))
 			    set_type (KEY_FORMAT_1, &(ih->ih_key), TYPE_INDIRECT);
 			else {
 			    set_type (KEY_FORMAT_1, &(ih->ih_key), TYPE_DIRECT);
@@ -1218,7 +1280,7 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
 	}
 */
 	ind_item = (__u32 *)B_I_PITEM (bh, ih);
-	for (j = 0; j < I_UNFM_NUM (ih); j ++) {
+	for (j = 0; j < (int)I_UNFM_NUM (ih); j ++) {
 	    unfm_ptr = le32_to_cpu (ind_item [j]);
 	    if (!unfm_ptr)
 		continue;
@@ -1257,11 +1319,11 @@ static void pass0_correct_leaf (reiserfs_filsys_t * fs,
     for (i = 0; i < get_blkh_nr_items (B_BLK_HEAD (bh)); i ++, ih ++) {
 	struct reiserfs_de_head * deh;
 
-	mark_objectid_really_used (proper_id_map (fs), get_key_dirid (&ih->ih_key));
-	mark_objectid_really_used (proper_id_map (fs), get_key_objectid (&ih->ih_key));
+	id_map_mark(proper_id_map (fs), get_key_dirid (&ih->ih_key));
+	id_map_mark(proper_id_map (fs), get_key_objectid (&ih->ih_key));
 	if (is_direntry_ih(ih)) {
 	    for (j = 0, deh = B_I_DEH (bh, ih); j < get_ih_entry_count (ih); j ++, deh++) 
-		mark_objectid_really_used(proper_id_map (fs), get_deh_objectid(deh));	    
+		id_map_mark(proper_id_map(fs), get_deh_objectid(deh));	    
 	}
     }
 
@@ -1331,24 +1393,23 @@ int is_bad_directory (struct item_head * ih, char * item, int dev, int blocksize
 
     for (i = 0; i < get_ih_entry_count (ih); i ++, deh ++) {
 	entrylen = entry_length(ih, deh, i);
-	if (entrylen > REISERFS_MAX_NAME_LEN (blocksize)) {
+	if (entrylen > (int)REISERFS_MAX_NAME_LEN (blocksize))
 	    return 1;
-	}
-	if (get_deh_offset (deh) <= prev_offset) {
+	
+	if (get_deh_offset (deh) <= prev_offset)
 	    return 1;
-	}
+	
 	prev_offset = get_deh_offset (deh);
 
-	if (get_deh_location(deh) + entrylen != prev_location) {
+	if (get_deh_location(deh) + entrylen != prev_location)
 	    return 1;
-	}
+	
 	prev_location = get_deh_location (deh);
 
 	namelen = name_in_entry_length (ih, deh, i);
 	name = name_in_entry (deh, i);
-	if (!is_properly_hashed (fs, name, namelen, get_deh_offset (deh))) {
+	if (!is_properly_hashed (fs, name, namelen, get_deh_offset (deh)))
 	    return 1;
-	}
     }
     return 0;
 }
@@ -1357,9 +1418,9 @@ int is_bad_directory (struct item_head * ih, char * item, int dev, int blocksize
 /* change incorrect block adresses by 0. Do not consider such item as incorrect */
 static int is_bad_indirect (struct item_head * ih, char * item, int dev, int blocksize)
 {
-    int i;
-    int bad = 0;
     unsigned long blocks;
+    unsigned int i;
+    int bad = 0;
 
     if (get_ih_item_len(ih) % UNFM_P_SIZE) {
 	fsck_log ("is_bad_indirect: indirect item of %H of invalid length\n", ih);
@@ -1509,7 +1570,7 @@ static void do_pass_0 (reiserfs_filsys_t * fs)
 
 	pass_0_stat (fs)->dealt_with ++;
 	what_node = who_is_this (bh->b_data, bh->b_size);
-	if ( what_node != THE_LEAF ) {
+	if ( what_node != THE_LEAF && what_node != HAS_IH_ARRAY ) {
 	    brelse (bh);
 	    continue;
 	}
@@ -1522,7 +1583,7 @@ static void do_pass_0 (reiserfs_filsys_t * fs)
 
 
     /* just in case */
-    mark_objectid_really_used (proper_id_map (fs), REISERFS_ROOT_OBJECTID);
+    id_map_mark(proper_id_map (fs), REISERFS_ROOT_OBJECTID);
 
 }
 
@@ -1592,16 +1653,16 @@ int still_bad_unfm_ptr_2 (unsigned long block)
 
 
 /* these are used to allocate blocks for tree building */
-int are_there_allocable_blocks (int amout_needed)
-{
+int are_there_allocable_blocks (unsigned int amout_needed) {
     if (reiserfs_bitmap_zeros (fsck_allocable_bitmap (fs)) < amout_needed) {
-	int zeros = 0, i;
+	unsigned int zeros = 0, i;
 	
 	fsck_progress ("Not enough allocable blocks, checking bitmap...");
-	for (i = 0; i < fsck_allocable_bitmap (fs)->bm_bit_size; i ++)
+	for (i = 0; i < fsck_allocable_bitmap (fs)->bm_bit_size; i ++) {
 	    if (!reiserfs_bitmap_test_bit (fsck_allocable_bitmap (fs), i))
 		zeros ++;
-	fsck_progress ("there are %d allocable blocks, btw\n", zeros);
+	}
+	fsck_progress ("there are %u allocable blocks, btw\n", zeros);
 	return 0;
     }
     return 1;
@@ -1629,7 +1690,7 @@ void make_allocable (unsigned long block)
 static void choose_hash_function (reiserfs_filsys_t * fs)
 {
     unsigned long max;
-    int hash_code;
+    unsigned int hash_code;
     int i;
 
     if (fsck_hash_defined (fs))
@@ -1798,7 +1859,7 @@ static void before_pass_0 (reiserfs_filsys_t * fs)
     make_aux_bitmaps (fs);
 
     /* on pass0 all objectids will be marked here as used */
-    proper_id_map (fs) = init_id_map (MAP_PACKED);
+    proper_id_map (fs) = id_map_init();
 
     /* pass0 gathers statistics about hash hits */
     hash_hits_init (fs);
@@ -1822,7 +1883,6 @@ static void save_pass_0_result (reiserfs_filsys_t * fs)
     reiserfs_bitmap_save (file,  bad_unfm_bitmap);
     reiserfs_end_stage_info_save (file);
     close_file (file);
-    retval = unlink (state_dump_file (fs));
     retval = rename ("temp_fsck_file.deleteme", state_dump_file (fs));
     if (retval != 0)
 	fsck_progress ("%s: Could not rename the temporary file temp_fsck_file.deleteme to %s",
@@ -1857,7 +1917,7 @@ static void after_pass_0 (reiserfs_filsys_t * fs)
 
     /* update super block: hash, objectid map, fsck state */
     choose_hash_function (fs);
-    flush_objectid_map (proper_id_map (fs), fs);
+    id_map_flush(proper_id_map (fs), fs);
     set_sb_fs_state (fs->fs_ondisk_sb, PASS_0_DONE);
     mark_buffer_dirty (fs->fs_super_bh);
 
@@ -1882,7 +1942,7 @@ static void after_pass_0 (reiserfs_filsys_t * fs)
     } else
 	save_pass_0_result (fs);
 
-    free_id_map (proper_id_map (fs));
+    id_map_free(proper_id_map (fs));
     proper_id_map (fs) = 0;
     
     time (&t);
