@@ -1,5 +1,6 @@
 /*
- * Copyright 1996-2003 Hans Reiser
+ * Copyright 1996-2003 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
 
 #include "fsck.h"
@@ -59,7 +60,7 @@ __u32 objectid_for_relocation (struct key * key)
     cur = getmem (sizeof (struct relocated));
     cur->old_dir_id = get_key_dirid (key);
     cur->old_objectid = get_key_objectid (key);
-    cur->new_objectid = get_unused_objectid (fs);
+    cur->new_objectid = id_map_alloc(proper_id_map(fs));
     cur->next = relocated_list;
     relocated_list = cur;
 /*    fsck_log ("relocation: (%K) is relocated to (%lu, %lu)\n",
@@ -443,7 +444,6 @@ static void save_pass_2_result (reiserfs_filsys_t * fs)
     reiserfs_begin_stage_info_save(file, TREE_IS_BUILT);
     reiserfs_end_stage_info_save (file);
     close_file (file);
-    retval = unlink (state_dump_file (fs));
     retval = rename ("temp_fsck_file.deleteme", state_dump_file (fs));
     if (retval != 0)
 	fsck_progress ("%s: Could not rename the temporary file temp_fsck_file.deleteme to %s",
@@ -466,8 +466,10 @@ void load_pass_2_result (reiserfs_filsys_t * fs)
     fs->block_deallocator = reiserfsck_reiserfs_free_block;
 
     /* we need objectid map on semantic pass to be able to relocate files */
-    proper_id_map (fs) = init_id_map (MAP_NOT_PACKED);
+    proper_id_map (fs) = id_map_init();
+    /* Not implemented yet.
     fetch_objectid_map (proper_id_map (fs), fs);    
+    */
 }
 
     
@@ -566,7 +568,7 @@ static void after_pass_2 (reiserfs_filsys_t * fs)
   
     /* write all dirty blocks */
     fsck_progress ("Flushing..");
-    flush_objectid_map (proper_id_map (fs), fs);
+    id_map_flush(proper_id_map (fs), fs);
     fs->fs_dirt = 1;
     reiserfs_flush_to_ondisk_bitmap (fs->fs_bitmap2, fs);
     reiserfs_flush (fs);
@@ -590,7 +592,7 @@ static void after_pass_2 (reiserfs_filsys_t * fs)
 	save_pass_2_result (fs);
 
     
-    free_id_map (proper_id_map (fs));
+    id_map_free(proper_id_map (fs));
     proper_id_map (fs) = 0;
     
     reiserfs_delete_bitmap (fsck_new_bitmap (fs));
@@ -620,7 +622,8 @@ void pass_2 (reiserfs_filsys_t * fs)
     
     after_pass_2 (fs);
 
-    if (get_sb_root_block (fs->fs_ondisk_sb) == ~0ul || get_sb_root_block (fs->fs_ondisk_sb) == 0)
+    if (get_sb_root_block (fs->fs_ondisk_sb) == ~(__u32)0 || 
+	get_sb_root_block (fs->fs_ondisk_sb) == 0)
 	die ( "\nNo reiserfs metadata found.  If you are sure that you had the reiserfs\n"
 		"on this partition,  then the start  of the partition  might be changed\n"
 		"or all data were wiped out. The start of the partition may get changed\n"
