@@ -1,6 +1,7 @@
 /*
- * Copyright 1996-2003 Hans Reiser
+ * Copyright 1996-2003 by Hans Reiser, licensing governed by reiserfsprogs/README
  */
+
 #include "fsck.h"
 
 static struct key *trunc_links = NULL;
@@ -9,7 +10,7 @@ static __u32 links_num = 0;
 int wrong_mode (struct key * key, __u16 * mode, __u64 real_size, int symlink);
 int wrong_st_blocks(struct key * key, __u32 * blocks, __u32 sd_blocks, __u16 mode, 
 		    int new_format);
-int wrong_st_size (struct key * key, loff_t max_file_size, int blocksize,
+int wrong_st_size (struct key * key, unsigned long long max_file_size, int blocksize,
 		   __u64 * size, __u64 sd_size, int type);
 int wrong_first_direct_byte (struct key * key, int blocksize, __u32 * first_direct_byte,
 			     __u32 sd_first_direct_byte, __u32 size);
@@ -97,7 +98,6 @@ static int check_check_regular_file (struct path * path, void * sd,
     __u32 blocks, sd_blocks;	/* proper values and value in stat data */
     __u32 first_direct_byte, sd_first_direct_byte;
 
-    struct buffer_head * bh;
     struct item_head * ih, sd_ih;
     int fix_sd;
     int symlnk = 0;
@@ -106,7 +106,6 @@ static int check_check_regular_file (struct path * path, void * sd,
 
 
     ih = get_ih (path);
-    bh = get_bh (path);
 
     if (new_ih) {
 	/* this objectid is used already */
@@ -122,7 +121,6 @@ static int check_check_regular_file (struct path * path, void * sd,
 		"the relocated file %K", &new_ih->ih_key);
 	/* stat data is marked unreachable again due to relocation, fix that */
 	ih = get_ih (path);
-	bh = get_bh (path);
 	sd = get_item (path);
     }
     
@@ -188,6 +186,7 @@ static int check_check_regular_file (struct path * path, void * sd,
 
 	if (fix_sd) {
 	    if (fsck_mode (fs) == FSCK_FIX_FIXABLE) {
+    		struct buffer_head * bh;
   	        /* find stat data and correct it */
   	        set_type_and_offset (KEY_FORMAT_1, &sd_ih.ih_key, SD_OFFSET, TYPE_STAT_DATA);
 	        if (reiserfs_search_by_key_4 (fs, &sd_ih.ih_key, path) != ITEM_FOUND) {
@@ -281,12 +280,12 @@ start_again:
     /* mark hidden entries as visible, set "." and ".." correctly */
     deh += *pos_in_item;
     for (i = *pos_in_item; i < get_ih_entry_count (ih); i ++, deh ++) {
-	int namelen;
+/*	int namelen;
 	char * name;
 
 	name = name_in_entry (deh, i);
 	namelen = name_in_entry_length (ih, deh, i);
-/*	if (de_hidden (deh)) // handled in check_tree
+	if (de_hidden (deh)) // handled in check_tree
 	    reiserfs_panic ("get_next_directory_item: item %k: hidden entry %d \'%.*s\'\n",
 			    key, i, namelen, name);
 */
@@ -339,7 +338,7 @@ start_again:
 	set_key_objectid (key, 0);
     }
 
-    if (fsck_mode (fs) != FSCK_CHECK && fsck_mode (fs) != FSCK_FIX_FIXABLE)
+    if (fsck_mode (fs) == FSCK_REBUILD)
         mark_item_reachable (get_ih (&path), bh);
     pathrelse (&path);
 
@@ -405,6 +404,7 @@ static int check_semantic_pass (struct key * key, struct key * parent, int dot_d
 
     if (fix_obviously_wrong_sd_mode (&path)) {
         one_more_corruption (fs, FIXABLE);
+	pathrelse (&path);
         return OK;
     }
     
@@ -740,7 +740,7 @@ int check_safe_links ()
 		if (reiserfs_bin_search (&key, trunc_links, links_num, sizeof(key), 
 		    &position, comp_short_keys) != POSITION_FOUND) 
 		{
-		    blocklist__insert_in_position(&key, (void **)&trunc_links, &links_num, 
+		    blocklist__insert_in_position(&key, (void *)&trunc_links, &links_num, 
 			sizeof(key), &position);
 		}		
 	    }
@@ -799,7 +799,7 @@ void semantic_check (void)
         fsck_allocable_bitmap (fs) = NULL;
         set_sb_free_blocks (fs->fs_ondisk_sb, reiserfs_bitmap_zeros (fs->fs_bitmap2));
         mark_buffer_dirty (fs->fs_super_bh);
-	add_badblock_list(fs, 0);
+	//add_badblock_list(fs, 0);
     }
     
     fsck_progress ("finished\n");
