@@ -1,6 +1,8 @@
 /*
- * Copyright 1996-2003 Hans Reiser
+ * Copyright 1996-2003 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
+
 #include "fsck.h"
 
 int screen_width;
@@ -93,8 +95,8 @@ void erase_name (int len)
 
 
 /* *size is "real" file size, sd_size - size from stat data */
-int wrong_st_size (struct key * key, loff_t max_file_size, int blocksize,
-		   __u64 * size, __u64 sd_size, int type)
+int wrong_st_size (struct key * key, unsigned long long max_file_size, 
+    int blocksize, __u64 * size, __u64 sd_size, int type)
 {
     if (sd_size <= max_file_size) {
 	if (sd_size == *size)
@@ -387,7 +389,6 @@ int rebuild_check_regular_file (struct path * path, void * sd,
     __u64 real_size, saved_size;
     __u32 blocks, saved_blocks;	/* proper values and value in stat data */
     __u32 first_direct_byte, saved_first_direct_byte;
-    __u32 pos_in_item;
 
     struct buffer_head * bh;
     struct item_head * ih, sd_ih;
@@ -420,10 +421,7 @@ int rebuild_check_regular_file (struct path * path, void * sd,
 	
     }
 	
-    if (!is_objectid_really_used(semantic_id_map (fs), get_key_objectid (&ih->ih_key), 
-				&pos_in_item))
-	__mark_objectid_really_used(semantic_id_map (fs), get_key_objectid (&ih->ih_key), 
-				    pos_in_item);
+    id_map_mark(semantic_id_map(fs), get_key_objectid (&ih->ih_key));
 
     /* check and set nlink first */
     get_sd_nlink (ih, sd, &nlink);
@@ -604,7 +602,7 @@ static char * get_next_directory_item (struct key * key, /* on return this will
 	set_key_objectid (key, 0);
     }
 
-    if (fsck_mode (fs) != FSCK_CHECK && fsck_mode (fs) != FSCK_FIX_FIXABLE)
+    if (fsck_mode (fs) == FSCK_REBUILD)
         mark_item_reachable (get_ih (&path), bh);
     pathrelse (&path);
 
@@ -654,8 +652,8 @@ int fix_obviously_wrong_sd_mode (struct path * path) {
 	    mark_buffer_dirty (get_bh(path));
 	} else {
 	    fsck_log ("\n");
+	    retval = 1;
 	}
-	retval = 1;
     } else if (!not_a_directory (get_item (path)) && !is_direntry_key (next_key)) {
         /* make SD mode SD of regular file */
 	get_sd_mode (get_ih (path), get_item (path), &mode);
@@ -668,9 +666,9 @@ int fix_obviously_wrong_sd_mode (struct path * path) {
 	    mark_buffer_dirty (get_bh(path));
 	} else {
 	    fsck_log ("\n");
+	    retval = 1;
 	}
 	    
-	retval = 1;
     }
 
     return retval;
@@ -733,18 +731,14 @@ int rebuild_semantic_pass (struct key * key, struct key * parent, int dot_dot,
     relocate = 0;
     if (!nlink) {
 	/* we reached the stat data for the first time */
-	if (is_objectid_really_used(semantic_id_map (fs), get_key_objectid (&ih->ih_key), 
-				    &pos_in_item)) 
-	{
+	if (id_map_mark(semantic_id_map(fs), get_key_objectid (&ih->ih_key))) {
 	    /* calculate number of found files/dirs who are using objectid
 	       which is used by another file */
 	    sem_pass_stat (fs)->oid_sharing ++;
 	    if (1/*fsck_adjust_file_size (fs)*/)
 		/* this works for files only */
 		relocate = 1;
-	} else
-	    __mark_objectid_really_used(semantic_id_map (fs), get_key_objectid (&ih->ih_key), 
-					pos_in_item);
+	} 
 
 	mark_item_reachable (ih, bh);
     }
@@ -1062,7 +1056,7 @@ static void make_sure_lost_found_exists (reiserfs_filsys_t * fs)
 {
     int retval;
     INITIALIZE_PATH (path);
-    int gen_counter;
+    unsigned int gen_counter;
     __u32 objectid;
     __u64 sd_size;
     __u32 sd_blocks;
@@ -1077,7 +1071,7 @@ static void make_sure_lost_found_exists (reiserfs_filsys_t * fs)
 				  "lost+found", &gen_counter,
 				  &lost_found_dir_key);
     if (!retval) {
-	objectid = get_unused_objectid (fs);
+	objectid = id_map_alloc(proper_id_map(fs));
 	if (!objectid) {
 	    fsck_progress ("Could not allocate an objectid for \"/lost+found\", \
 		lost files will not be linked\n");
@@ -1153,10 +1147,11 @@ static void save_rebuild_semantic_result (reiserfs_filsys_t * fs) {
 	return;
 
     reiserfs_begin_stage_info_save (file, SEMANTIC_DONE);
+    /*  Method not implemented yet.
     reiserfs_objectid_map_save (file, semantic_id_map (fs));
+    */
     reiserfs_end_stage_info_save (file);
     close_file (file);
-    retval = unlink (state_dump_file (fs));
     retval = rename ("temp_fsck_file.deleteme", state_dump_file (fs));
     if (retval != 0)
 	fsck_progress ("%s: Could not rename the temporary file temp_fsck_file.deleteme to %s",
@@ -1178,14 +1173,16 @@ void load_semantic_result (FILE * file, reiserfs_filsys_t * fs)
     fs->block_deallocator = reiserfsck_reiserfs_free_block;
 
     /* we need objectid map on semantic pass to be able to relocate files */
-    proper_id_map (fs) = init_id_map (MAP_NOT_PACKED);
+    proper_id_map (fs) = id_map_init();
+    /* Not implemented yet.    
     fetch_objectid_map (proper_id_map (fs), fs);
     semantic_id_map (fs) = reiserfs_objectid_map_load (file);
+    */
 }
 
 static void before_pass_3 (reiserfs_filsys_t * fs)
 {
-    semantic_id_map (fs) = init_id_map (MAP_NOT_PACKED);
+    semantic_id_map (fs) = id_map_init();
 }
 
 static void after_pass_3 (reiserfs_filsys_t * fs)
@@ -1196,7 +1193,7 @@ static void after_pass_3 (reiserfs_filsys_t * fs)
 
     /* write all dirty blocks */
     fsck_progress ("Flushing..");
-    flush_objectid_map (proper_id_map (fs), fs);
+    id_map_flush(proper_id_map (fs), fs);
     fs->fs_dirt = 1;
     reiserfs_flush_to_ondisk_bitmap (fsck_new_bitmap(fs), fs);
     reiserfs_flush (fs);
@@ -1212,7 +1209,7 @@ static void after_pass_3 (reiserfs_filsys_t * fs)
 
     save_rebuild_semantic_result (fs);
 
-    free_id_map (proper_id_map (fs));
+    id_map_free(proper_id_map (fs));
     proper_id_map (fs) = 0;
 
     fs->fs_dirt = 1;
@@ -1227,7 +1224,7 @@ void pass_3_semantic (reiserfs_filsys_t * fs)
 
     fsck_progress ("Pass 3 (semantic):\n");
 
-    /* when warnings go not to stderr - separate then in the log */
+    /* when warnings go not to stderr - separate them in the log */
     if (fsck_log_file (fs) != stderr)
 	fsck_log ("####### Pass 3 #########\n");
 
@@ -1238,15 +1235,15 @@ void pass_3_semantic (reiserfs_filsys_t * fs)
     make_sure_root_dir_exists (fs, modify_item, 1 << IH_Unreachable);
     make_sure_lost_found_exists (fs);
 
-    mark_objectid_really_used (proper_id_map (fs), get_key_objectid (&root_dir_key));
-    mark_objectid_really_used (proper_id_map (fs), get_key_objectid (&lost_found_dir_key));
+    id_map_mark(proper_id_map(fs), get_key_objectid(&root_dir_key));
+    id_map_mark(proper_id_map(fs), get_key_objectid(&lost_found_dir_key));
     
     /* link all relocated files into /lost+found directory */
     link_relocated_files ();
 
     rebuild_semantic_pass (&root_dir_key, &parent_root_dir_key, 0/*!dot_dot*/, 0/*reloc_ih*/);
 
-    add_badblock_list(fs, 1);
+    //add_badblock_list(fs, 1);
 
     after_pass_3 (fs);
 }
