@@ -1,8 +1,8 @@
 /*
- * Copyright 1996-2003 Hans Reiser
+ * Copyright 1996-2003 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
 
-
 struct buffer_head {
     unsigned long b_blocknr;
     int b_dev;
@@ -59,15 +59,14 @@ int valid_offset (int fd, loff_t offset);
 int bwrite (struct buffer_head * bh);
 void brelse (struct buffer_head * bh);
 void bforget (struct buffer_head * bh);
-void check_and_free_buffer_mem (void);
 void init_rollback_file (char * rollback_file, int *bloksize, FILE * log);
 int open_rollback_file (char * rollback_file, FILE * log);
 void close_rollback_file ();
 void do_fsck_rollback (int fd_device, int fd_journal_device, FILE * log); 
 
-void flush_buffers (dev_t);
+void flush_buffers (int);
 void free_buffers (void);
-void invalidate_buffers (dev_t dev);
+void invalidate_buffers (int);
 
 
 int user_confirmed (FILE * fp, char * q, char * yes);
