@@ -1,5 +1,6 @@
 /*
- *  Copyright 2000-2003 by Hans Reiser, licensing governed by reiserfs/README
+ *  Copyright 2000-2003 by Hans Reiser, licensing governed by 
+ *  reiserfsprogs/README
  */
 
 #include "includes.h"
@@ -26,13 +27,13 @@ static void make_const_keys (void)
 /* reiserfs needs at least: enough blocks for journal, 64 k at the beginning,
    one block for super block, bitmap block and root block. Note that first
    bitmap block must point to all of them */
-int is_block_count_correct (unsigned long block_of_super_block, int block_size,
-	unsigned long block_count, unsigned long journal_size)
+int is_block_count_correct (unsigned long journal_offset, unsigned int block_size,
+    unsigned long block_count, unsigned long journal_size)
 {
     unsigned long blocks;
 
     /* RESERVED, MD RAID SBs, super block, bitmap, root, journal size with journal header */
-    blocks = block_of_super_block + 1 + 1 + 1 + journal_size;
+    blocks = journal_offset + journal_size;
 
     /* we have a limit: skipped area, super block, journal and root block
     all have to be addressed by one first bitmap */
@@ -48,7 +49,7 @@ int is_block_count_correct (unsigned long block_of_super_block, int block_size,
 /* read super block. fixme: only 4k blocks, pre-journaled format
    is refused. Journal and bitmap are to be opened separately.
    skip_check is set to 1 if checks of openned SB should be omitted.*/
-reiserfs_filsys_t * reiserfs_open (char * filename, int flags, int *error, void * vp, int skip_check)
+reiserfs_filsys_t * reiserfs_open (char * filename, int flags, int *error, void * vp, int check)
 {
     reiserfs_filsys_t * fs;
     struct buffer_head * bh;
@@ -100,7 +101,7 @@ reiserfs_filsys_t * reiserfs_open (char * filename, int flags, int *error, void
 
  found:
 
-    if (!skip_check) {
+    if (check) {
 	/* A few checks of found super block. */
 	struct buffer_head *tmp_bh;
 	
@@ -166,7 +167,10 @@ reiserfs_filsys_t * reiserfs_open (char * filename, int flags, int *error, void
    constant for given size and version of a filesystem */
 reiserfs_filsys_t * reiserfs_create (char * filename,
 				     int version,
-				     unsigned long block_count, int block_size, int default_journal, int new_format)
+				     unsigned long block_count, 
+				     int block_size, 
+				     int default_journal, 
+				     int new_format)
 {
     reiserfs_filsys_t * fs;
 
@@ -180,11 +184,11 @@ reiserfs_filsys_t * reiserfs_create (char * filename,
 	return 0;
     }
 
-    if (!is_block_count_correct (REISERFS_DISK_OFFSET_IN_BYTES / block_size, block_size, block_count, 0)) {
-//    if (block_count < min_block_amount (block_size, 0)) {
-	reiserfs_warning (stderr, "reiserfs_create: "
-			  "can not create that small (%d blocks) filesystem\n",
-			  block_count);
+    if (!is_block_count_correct (REISERFS_DISK_OFFSET_IN_BYTES / block_size, 
+	block_size, block_count, 0)) 
+    {
+	reiserfs_warning (stderr, "reiserfs_create: can not create that small "
+	    "(%d blocks) filesystem\n", block_count);
 	return 0;
     }
 
@@ -207,9 +211,11 @@ reiserfs_filsys_t * reiserfs_create (char * filename,
     fs->fs_format = version;
 
     if (new_format)
-        fs->fs_super_bh = getblk (fs->fs_dev, REISERFS_DISK_OFFSET_IN_BYTES / block_size, block_size);
+        fs->fs_super_bh = getblk (fs->fs_dev, 
+	    REISERFS_DISK_OFFSET_IN_BYTES / block_size, block_size);
     else 
-        fs->fs_super_bh = getblk (fs->fs_dev, REISERFS_OLD_DISK_OFFSET_IN_BYTES / block_size, block_size);
+        fs->fs_super_bh = getblk (fs->fs_dev, 
+	    REISERFS_OLD_DISK_OFFSET_IN_BYTES / block_size, block_size);
     
     if (!fs->fs_super_bh) {
 	reiserfs_warning (stderr, "reiserfs_create: getblk failed\n");
@@ -236,7 +242,8 @@ reiserfs_filsys_t * reiserfs_create (char * filename,
     set_sb_block_size (fs->fs_ondisk_sb, block_size);
     switch (version) {
     case REISERFS_FORMAT_3_5:
-	set_sb_oid_maxsize (fs->fs_ondisk_sb, (block_size - SB_SIZE_V1) / sizeof(__u32) / 2 * 2);
+	set_sb_oid_maxsize (fs->fs_ondisk_sb, 
+	    (block_size - SB_SIZE_V1) / sizeof(__u32) / 2 * 2);
 	/* sb_oid_cursize */
 	/* sb_state */
 	memcpy (fs->fs_ondisk_sb->s_v1.s_magic, REISERFS_3_5_SUPER_MAGIC_STRING,
@@ -244,7 +251,8 @@ reiserfs_filsys_t * reiserfs_create (char * filename,
 	break;
 
     case REISERFS_FORMAT_3_6:
-	set_sb_oid_maxsize (fs->fs_ondisk_sb, (block_size - SB_SIZE) / sizeof(__u32) / 2 * 2);
+	set_sb_oid_maxsize (fs->fs_ondisk_sb, 
+	    (block_size - SB_SIZE) / sizeof(__u32) / 2 * 2);
 	/* sb_oid_cursize */
 	/* sb_state */
         memcpy (fs->fs_ondisk_sb->s_v1.s_magic, REISERFS_3_6_SUPER_MAGIC_STRING,
@@ -385,7 +393,9 @@ void reiserfs_close (reiserfs_filsys_t * fs)
 
 
 int reiserfs_new_blocknrs (reiserfs_filsys_t * fs, 
-			   unsigned long * free_blocknrs, unsigned long start, int amount_needed)
+			   unsigned long * free_blocknrs, 
+			   unsigned long start, 
+			   int amount_needed)
 {
     if (fs->block_allocator)
 	return fs->block_allocator (fs, free_blocknrs, start, amount_needed);
@@ -883,7 +893,7 @@ int reiserfs_locate_entry (reiserfs_filsys_t * fs, struct key * dir, char * name
 	    }
 
 	    /* the name in directory has the same hash as the given name */
-	    if ((name_in_entry_length (ih, deh, i) == strlen (name)) &&
+	    if ((name_in_entry_length (ih, deh, i) == (int)strlen (name)) &&
 		!memcmp (name_in_entry (deh, i), name, strlen (name))) {
 		path->pos_in_item = i;
 		return 1;
@@ -925,7 +935,7 @@ int reiserfs_locate_entry (reiserfs_filsys_t * fs, struct key * dir, char * name
    generation counter in 'min_gen_counter'. dies if found object is not a
    directory. */
 int reiserfs_find_entry (reiserfs_filsys_t * fs, struct key * dir, char * name, 
-			 int * min_gen_counter, struct key * key)
+			 unsigned int * min_gen_counter, struct key * key)
 {
     struct key entry_key;
     int retval;
@@ -963,7 +973,7 @@ int reiserfs_find_entry (reiserfs_filsys_t * fs, struct key * dir, char * name,
 	    if (GET_GENERATION_NUMBER (get_deh_offset (deh)) == *min_gen_counter)
 		(*min_gen_counter) ++;
 			
-	    if ((name_in_entry_length (ih, deh, i) == strlen (name)) &&
+	    if ((name_in_entry_length (ih, deh, i) == (int)strlen (name)) &&
 	        (!memcmp (name_in_entry (deh, i), name, strlen (name)))) {
 		/* entry found in the directory */
 		if (key) {
@@ -1040,7 +1050,7 @@ int reiserfs_add_entry (reiserfs_filsys_t * fs, struct key * dir, char * name, i
     char * entry;
     int retval;
     INITIALIZE_PATH(path);
-    int gen_counter;
+    unsigned int gen_counter;
     int item_len;
     __u32 hash;
 
@@ -1133,6 +1143,7 @@ int create_dir_sd (reiserfs_filsys_t * fs,
     else
 	key_format = KEY_FORMAT_2;
 
+    memset(&sd, 0, sizeof(sd));
     make_dir_stat_data (fs->fs_blocksize, key_format, get_key_dirid (key),
 			get_key_objectid (key), &ih, &sd);
 
@@ -1242,7 +1253,8 @@ int create_badblock_bitmap (reiserfs_filsys_t * fs, char * badblocks_file) {
 	if (blocknr < get_sb_block_count (fs->fs_ondisk_sb) && !not_data_block (fs, blocknr)) {
 	    reiserfs_bitmap_set_bit (fs->fs_badblocks_bm, blocknr);
 	} else {
-	    fprintf (stderr, "%s: bad block number %u in badblocks file\n", __FUNCTION__, blocknr);
+	    fprintf (stderr, "%s: block number %u belongs to internal reiserfs structures.\n", 
+		__FUNCTION__, blocknr);
 	}
     }
 
