@@ -1,6 +1,8 @@
 /* 
- * Copyright 2000-2003 by Hans Reiser, licensing governed by reiserfs/README
+ * Copyright 2000-2003 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
+
 #define _GNU_SOURCE
 
 #include <sys/types.h>
@@ -29,7 +31,7 @@
 
 
 #define print_usage_and_exit() {\
- fprintf (stderr, "Usage: %s  [-s[+|-]#[G|M|K]] [-fqv] device\n\n", argv[0]);\
+ fprintf (stderr, "Usage: %s  [-s[+|-]#[G|M|K]] [-fqvV] device\n\n", argv[0]);\
  exit(16);\
 }
 
