@@ -13,6 +13,7 @@
 #include "resize.h"
 #include <libgen.h>
 
+int opt_banner = 0;
 int opt_force = 0;
 int opt_verbose = 1;			/* now "verbose" option is default */
 int opt_nowrite = 0;
@@ -147,9 +148,11 @@ int main(int argc, char *argv[]) {
     unsigned long block_count_new;
 
     g_progname = basename(argv[0]);
-    print_banner (g_progname);
+    
+    if (argc < 2)
+	print_usage_and_exit();
 	
-    while ((c = getopt(argc, argv, "fvcqks:j:")) != EOF) {
+    while ((c = getopt(argc, argv, "fvcqks:j:V")) != EOF) {
 	switch (c) {
 	case 's' :
 	    if (!optarg)
@@ -179,13 +182,20 @@ int main(int argc, char *argv[]) {
 	case 'k':
 	    opt_skipj = 1;
 	    break;
+	case 'V':
+	    opt_banner++;
+	    break;
 	default:
 	    print_usage_and_exit ();
 	}
     }
 
-    if (optind == argc )
-	print_usage_and_exit();
+    print_banner (g_progname);
+    
+    if (opt_banner)
+	exit(0);
+    
+
     devname = argv[optind];
 
     fs = reiserfs_open(devname, O_RDONLY, &error, 0, 1);
